@@ -41,6 +41,9 @@
 #include <unistd.h>
 #include <sys/wait.h>
 
+#define MONOSPACE_FONT_DIR "/desktop/gnome/interface"
+#define MONOSPACE_FONT_KEY MONOSPACE_FONT_DIR "/monospace_font_name"
+
 struct _TerminalScreenPrivate
 {
   GtkWidget *term;
@@ -60,6 +63,7 @@ struct _TerminalScreenPrivate
   int child_pid;
   double font_scale;
   guint recheck_working_dir_idle;
+  guint gconf_connection_id;
 };
 
 static GList* used_ids = NULL;
@@ -103,12 +107,16 @@ static void rebuild_title  (TerminalScreen *screen);
 
 static void queue_recheck_working_dir (TerminalScreen *screen);
 
+static void monospace_font_change_notify (GConfClient *client,
+					  guint        cnxn_id,
+					  GConfEntry  *entry,
+					  gpointer     user_data);
+
 static gboolean xfont_is_monospace                   (const char *fontname);
 static char*    make_xfont_monospace                 (const char *fontname);
 static char*    make_xfont_char_cell                 (const char *fontname);
 static char*    make_xfont_have_size_from_other_font (const char *fontname,
                                                      const char *other_font);
-static PangoFontDescription* make_font_monospace (const PangoFontDescription *font);
 
 static GdkFont* load_fonset_without_error (const gchar *fontset_name);
 
@@ -155,6 +163,46 @@ style_set_callback (GtkWidget *widget,
     terminal_screen_update_on_realize (widget, TERMINAL_SCREEN (data));  
 }
 
+static void
+connect_monospace_font_change (TerminalScreen *screen)
+{
+  GError *err;
+  GConfClient *conf;
+  guint connection_id;
+
+  conf = gconf_client_get_default ();
+  
+  err = NULL;
+  gconf_client_add_dir (conf, MONOSPACE_FONT_DIR,
+                        GCONF_CLIENT_PRELOAD_ONELEVEL,
+                        &err);
+
+  if (err)
+    {
+      g_printerr (_("There was an error loading config from %s. (%s)\n"),
+                  MONOSPACE_FONT_DIR, err->message);
+      g_error_free (err);
+    }
+
+  err = NULL;
+  connection_id = gconf_client_notify_add (conf,
+					   MONOSPACE_FONT_DIR,
+					   monospace_font_change_notify,
+					   screen, 
+					   NULL, &err);
+
+  g_object_unref (conf);
+  
+  if (err)
+    {
+      g_printerr (_("There was an error subscribing to notification of monospace font changes. (%s)\n"),
+                  err->message);
+      g_error_free (err);
+    }
+  else
+    screen->priv->gconf_connection_id = connection_id;
+}
+
 static void
 terminal_screen_init (TerminalScreen *screen)
 {
@@ -176,11 +224,20 @@ terminal_screen_init (TerminalScreen *screen)
   g_object_ref (G_OBJECT (screen->priv->term));
   gtk_object_sink (GTK_OBJECT (screen->priv->term));
 
+#define USERCHARS "-A-Za-z0-9"
+#define PASSCHARS "-A-Za-z0-9,?;.:/!%$^*&~\"#'"
+#define HOSTCHARS "-A-Za-z0-9"
+#define SCHEME    "(news|telnet|nttp|file|http|ftp|https)"
+#define USER      "[" USERCHARS "]+(:["PASSCHARS "]+)?"
+
   terminal_widget_match_add (screen->priv->term,
-                             "(((news|telnet|nttp|file|http|ftp|https)://)|(www|ftp)[-A-Za-z0-9]*\\.)[-A-Za-z0-9\\.]+(:[0-9]*)?");
+      "((" SCHEME "://(" USER "@)?)|(www|ftp)[" HOSTCHARS "]*\\.)[" HOSTCHARS ".]+(:[0-9]*)?");
   
   terminal_widget_match_add (screen->priv->term,
-                             "(((news|telnet|nttp|file|http|ftp|https)://)|(www|ftp)[-A-Za-z0-9]*\\.)[-A-Za-z0-9\\.]+(:[0-9]*)?/[-A-Za-z0-9_\\$\\.\\+\\!\\*\\(\\),;:@&=\\?/~\\#\\%]*[^]'\\.}>\\) ,\\\"]");
+      "((" SCHEME "://(" USER "@)?)"  "|"  "(www|ftp)[" HOSTCHARS "]*\\.)"
+      "[" HOSTCHARS ".]+(:[0-9]+)?/"
+      "[-A-Za-z0-9_$.+!*(),;:@&=?/~#%]*"
+      "[^]'.}>) \t\r\n,\\\"]");
 
   terminal_screen_setup_dnd (screen);
   
@@ -224,6 +281,8 @@ terminal_screen_init (TerminalScreen *screen)
                                             G_CALLBACK (terminal_screen_widget_encoding_changed),
                                             screen);
 
+  connect_monospace_font_change (screen);
+
   gtk_widget_show (screen->priv->term);
 }
 
@@ -277,9 +336,20 @@ static void
 terminal_screen_finalize (GObject *object)
 {
   TerminalScreen *screen;
+  GConfClient *conf;
 
   screen = TERMINAL_SCREEN (object);
+
+  conf = gconf_client_get_default ();
   
+  if (screen->priv->gconf_connection_id)
+    gconf_client_notify_remove (conf, screen->priv->gconf_connection_id);
+  
+  gconf_client_remove_dir (conf, MONOSPACE_FONT_DIR,
+			   NULL);
+
+  g_object_unref (conf);
+
   used_ids = g_list_remove (used_ids, GINT_TO_POINTER (screen->priv->id));  
 
   if (screen->priv->title_editor)
@@ -410,7 +480,7 @@ reread_profile (TerminalScreen *screen)
   if (terminal_profile_get_use_skey (screen->priv->profile))
     {
       terminal_widget_skey_match_add (screen->priv->term,
-				      "s/key ([0-9]*) ([-A-Za-z0-9]*)");
+				      "s/key [0-9]* [-A-Za-z0-9]*");
     }
   else
     {
@@ -535,6 +605,42 @@ update_color_scheme (TerminalScreen *screen)
 /* Note this can be called on style_set, on prefs changing,
  * and on setting the font scale factor
  */
+static void
+monospace_font_change_notify (GConfClient *client,
+			      guint        cnxn_id,
+			      GConfEntry  *entry,
+			      gpointer     user_data)
+{
+  TerminalScreen *screen = TERMINAL_SCREEN (user_data);
+  GtkWidget *widget = screen->priv->term;
+  
+  if (strcmp (entry->key, MONOSPACE_FONT_KEY) == 0 &&
+      GTK_WIDGET_REALIZED (widget))
+    terminal_screen_update_on_realize (widget, screen);
+}
+
+PangoFontDescription *
+get_system_monospace_font (void)
+{
+  GConfClient *conf;
+  char *name;
+  PangoFontDescription *desc = NULL;
+
+  conf = gconf_client_get_default ();
+  name = gconf_client_get_string (conf, MONOSPACE_FONT_KEY, NULL);
+
+  if (name)
+    {
+      desc = pango_font_description_from_string (name);
+      g_free (name);
+    }
+  
+  if (!desc)
+    desc = pango_font_description_from_string ("Monospace 10");
+
+  return desc;
+}
+
 static void
 terminal_screen_update_on_realize (GtkWidget      *term,
                                    TerminalScreen *screen)
@@ -547,53 +653,20 @@ terminal_screen_update_on_realize (GtkWidget      *term,
 
   if (terminal_widget_supports_pango_fonts ())
     {
+      PangoFontDescription *desc;
+
       if (terminal_profile_get_use_system_font (profile))
-        {
-          PangoFontDescription *desc;
-
-          /* Try creating monospace variant of system font */
-          desc = make_font_monospace (term->style->font_desc);
-
-          if (desc)
-            {
-              terminal_widget_set_pango_font (term, desc);
-              
-              pango_font_description_free (desc);
-            }
-          else
-            {
-              /* At least use the same size */
-              desc = pango_font_description_copy (terminal_profile_get_font (profile));
-
-              if (pango_font_description_get_set_fields (term->style->font_desc) &
-                  PANGO_FONT_MASK_SIZE)
-                {
-                  pango_font_description_set_size (desc,
-                                                   pango_font_description_get_size (term->style->font_desc));
-                }
-
-              pango_font_description_set_size (desc,
-                                               screen->priv->font_scale *
-                                               pango_font_description_get_size (desc));
-              
-              terminal_widget_set_pango_font (term, desc);
-
-              pango_font_description_free (desc);
-            }
-        }
+        desc = get_system_monospace_font ();
       else
-        {
-          PangoFontDescription *desc;
+        desc = pango_font_description_copy (terminal_profile_get_font (profile));
 
-          desc = pango_font_description_copy (terminal_profile_get_font (profile));
-          pango_font_description_set_size (desc,
-                                           screen->priv->font_scale *
-                                           pango_font_description_get_size (desc));          
-          
-          terminal_widget_set_pango_font (term, desc);
+      pango_font_description_set_size (desc,
+                                       screen->priv->font_scale *
+                                       pango_font_description_get_size (desc));
 
-          pango_font_description_free (desc);
-        }
+      terminal_widget_set_pango_font (term, desc);
+
+      pango_font_description_free (desc);
     }
   else
     {
@@ -605,11 +678,11 @@ terminal_screen_update_on_realize (GtkWidget      *term,
       
        if (terminal_profile_get_use_system_font (screen->priv->profile))
          {
-            GtkStyle *style;
+	    PangoFontDescription *desc;
             GdkFont *from_desc;
 
-            style = gtk_widget_get_style (GTK_WIDGET(term));
-            from_desc = gdk_font_from_description (style->font_desc);
+	    desc = get_system_monospace_font ();
+            from_desc = gdk_font_from_description (desc);
             
             if (from_desc != NULL)
               {
@@ -652,6 +725,8 @@ terminal_screen_update_on_realize (GtkWidget      *term,
                 g_free (font_name);
                 gdk_font_unref (from_desc);
               }
+
+	    pango_font_description_free (desc);
          }
        
        if (font == NULL)
@@ -780,51 +855,14 @@ terminal_screen_get_widget (TerminalScreen *screen)
   return screen->priv->term;
 }
 
-static void
-show_fork_error_dialog (TerminalScreen *screen,
-                        GError         *err)
-{
-  GtkWidget *dialog;
-  
-  dialog = gtk_message_dialog_new ((GtkWindow*)
-                                   gtk_widget_get_ancestor (screen->priv->term,
-                                                            GTK_TYPE_WINDOW),
-                                   GTK_DIALOG_DESTROY_WITH_PARENT,
-                                   GTK_MESSAGE_ERROR,
-                                   GTK_BUTTONS_CLOSE,
-                                   "%s", err->message);
-
-  g_signal_connect (G_OBJECT (dialog),
-                    "response",
-                    G_CALLBACK (gtk_widget_destroy),
-                    NULL);
-
-  gtk_widget_show (dialog);
-}
-
 static void
 show_command_error_dialog (TerminalScreen *screen,
-                        GError         *error)
+                           GError         *error)
 {
-  GtkWidget *dialog;
-
   g_return_if_fail (error != NULL);
   
-  dialog = gtk_message_dialog_new ((GtkWindow*)
-                                   gtk_widget_get_ancestor (screen->priv->term,
-                                                            GTK_TYPE_WINDOW),
-                                   GTK_DIALOG_DESTROY_WITH_PARENT,
-                                   GTK_MESSAGE_ERROR,
-                                   GTK_BUTTONS_CLOSE,
-                                   _("There was a problem with the command for this terminal: %s"),
-                                   error->message);
-
-  g_signal_connect (G_OBJECT (dialog),
-                    "response",
-                    G_CALLBACK (gtk_widget_destroy),
-                    NULL);
-
-  gtk_widget_show (dialog);
+  terminal_util_show_error_dialog ((GtkWindow*) gtk_widget_get_ancestor (screen->priv->term, GTK_TYPE_WINDOW), NULL,
+                                   _("There was a problem with the command for this terminal: %s"), error->message);
 }
 
 static gboolean
@@ -991,7 +1029,9 @@ terminal_screen_launch_child (TerminalScreen *screen)
                                      &screen->priv->child_pid,
                                      &err))
     {
-      show_fork_error_dialog (screen, err);
+
+      terminal_util_show_error_dialog ((GtkWindow*) gtk_widget_get_ancestor (screen->priv->term, GTK_TYPE_WINDOW), NULL,
+                                       "%s", err->message);
       g_error_free (err);
     }
   
@@ -1117,7 +1157,6 @@ open_url (TerminalScreen *screen,
           const char     *orig_url)
 {
   GError *err;
-  GtkWidget *dialog;
   char *url;
   
   g_return_if_fail (orig_url != NULL);
@@ -1139,21 +1178,11 @@ open_url (TerminalScreen *screen,
       else
         window = NULL;
       
-      dialog = gtk_message_dialog_new (window ? GTK_WINDOW (window) : NULL,
-                                       GTK_DIALOG_DESTROY_WITH_PARENT,
-                                       GTK_MESSAGE_ERROR,
-                                       GTK_BUTTONS_CLOSE,
+      terminal_util_show_error_dialog (window ? GTK_WINDOW (window) : NULL, NULL,
                                        _("Could not open the address \"%s\":\n%s"),
                                        url, err->message);
       
       g_error_free (err);
-      
-      g_signal_connect (G_OBJECT (dialog),
-                        "response",
-                        G_CALLBACK (gtk_widget_destroy),
-                        NULL);
-      
-      gtk_widget_show (dialog);
     }
 
   g_free (url);
@@ -1290,6 +1319,24 @@ terminal_screen_do_popup (TerminalScreen *screen,
                              GTK_WIDGET (screen->priv->term),
                              popup_menu_detach);
 
+  if (screen->priv->matched_string != NULL) 
+    {
+      menu_item = append_menuitem (screen->priv->popup_menu,
+                                   _("_Open Link"),
+                                   G_CALLBACK (open_url_callback),
+                                   screen);
+      
+      menu_item = append_menuitem (screen->priv->popup_menu,
+                                   _("_Copy Link Address"),
+                                   G_CALLBACK (copy_url_callback),
+                                   screen);
+  
+      menu_item = gtk_separator_menu_item_new ();
+      gtk_widget_show (menu_item);
+      gtk_menu_shell_append (GTK_MENU_SHELL (screen->priv->popup_menu),
+                             menu_item);
+    }
+ 
   menu_item = append_menuitem (screen->priv->popup_menu,
                                _("_New Window"),
                                G_CALLBACK (new_window_callback),
@@ -1412,21 +1459,7 @@ terminal_screen_do_popup (TerminalScreen *screen,
                    G_CALLBACK (secure_keyboard_callback),
                    screen);
 #endif
-
-  menu_item = append_menuitem (screen->priv->popup_menu,
-                               _("_Open Link"),
-                               G_CALLBACK (open_url_callback),
-                               screen);
-
-  gtk_widget_set_sensitive (menu_item, screen->priv->matched_string != NULL);
-  
-  menu_item = append_menuitem (screen->priv->popup_menu,
-                               _("_Copy Link Address"),
-                               G_CALLBACK (copy_url_callback),
-                               screen);
-
-  gtk_widget_set_sensitive (menu_item, screen->priv->matched_string != NULL);
-  
+ 
   gtk_menu_popup (GTK_MENU (screen->priv->popup_menu),
                   NULL, NULL,
                   NULL, NULL, 
@@ -2334,19 +2367,6 @@ make_xfont_have_size_from_other_font (const char *fontname,
   return ret;
 }
 
-static PangoFontDescription*
-make_font_monospace (const PangoFontDescription *font)
-{
-  PangoFontDescription *ret;
-
-  ret = pango_font_description_copy (font);
-
-  /* FIXME */
-  pango_font_description_set_family (ret, "monospace");
-  
-  return ret;
-}
-
 static GdkFont*
 load_fonset_without_error (const gchar *fontset_name)
 {
