@@ -73,6 +73,10 @@ struct _TerminalWindowPrivate
   void *old_geometry_widget; /* only used for pointer value as it may be freed */
   GConfClient *conf;
   guint notify_id;
+  guint copy_notify_id;
+  guint paste_notify_id;
+  guint help_notify_id;
+  guint about_notify_id;
   char *startup_id;
   guint menubar_visible : 1;
   guint use_default_menubar_visibility : 1;
@@ -91,6 +95,9 @@ enum {
   LAST_SIGNAL
 };
 
+static GConfClient *client = NULL;
+static guint notify;
+
 static void terminal_window_init        (TerminalWindow      *window);
 static void terminal_window_class_init  (TerminalWindowClass *klass);
 static void terminal_window_finalize    (GObject             *object);
@@ -122,6 +129,14 @@ static void config_change_notify            (GConfClient *client,
                                              GConfEntry  *entry,
                                              gpointer     user_data);
 
+static void menuitem_icon_visibility_notify (GConfClient *client,
+                                             guint        cnxn_id,
+                                             GConfEntry  *entry,
+                                             gpointer     user_data);
+
+static void menuitem_icon_visibility        (GtkWidget *image,
+                                             gboolean   use_image);
+
 static void reset_menubar_labels          (TerminalWindow *window);
 static void reset_tab_menuitems           (TerminalWindow *window);
 
@@ -251,6 +266,28 @@ append_menuitem (GtkWidget  *menu,
   return menu_item;
 }
 
+static void
+menuitem_icon_visibility (GtkWidget *image, gboolean use_image)
+{
+  if (use_image)
+    gtk_widget_show (image);
+  else
+    gtk_widget_hide (image);
+}
+
+static void
+menuitem_icon_visibility_notify (GConfClient *client,
+                                 guint        cnxn_id,
+                                 GConfEntry  *entry,
+                                 gpointer     user_data)
+{
+  GConfValue *value = gconf_entry_get_value (entry);
+  
+  if (value && value->type == GCONF_VALUE_BOOL)
+    menuitem_icon_visibility (user_data, gconf_value_get_bool (value));
+}
+
+
 static GtkWidget*
 append_stock_menuitem (GtkWidget  *menu,
                        const char *text,
@@ -259,8 +296,25 @@ append_stock_menuitem (GtkWidget  *menu,
                        gpointer    data)
 {
   GtkWidget *menu_item;
-  
+  GtkWidget *image;
+  GError *error;
+
   menu_item = gtk_image_menu_item_new_from_stock (text, NULL);
+  image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (menu_item));
+
+  error = NULL;
+  notify = 0;
+
+  notify = gconf_client_notify_add (client, "/desktop/gnome/interface/menus_have_icons",
+                           menuitem_icon_visibility_notify,
+                           image,
+                           NULL, &error);
+  if (error)
+    {
+      g_printerr (_("There was an error subscribing to notification of menu icon visibility changes. (%s)\n"), error->message);
+      g_error_free (error);
+    }
+
   if (accel_path)
     gtk_menu_item_set_accel_path (GTK_MENU_ITEM (menu_item),
                                   accel_path);
@@ -343,6 +397,28 @@ fill_in_config_picker_submenu (TerminalWindow *window)
   g_list_free (profiles);  
 }
 
+static char *
+escape_underscores (const char *name)
+{
+  GString *escaped_name;
+
+  g_assert (name != NULL);
+
+  /* Who'd use more that 4 underscores in a profile name... */
+  escaped_name = g_string_sized_new (strlen (name) + 4 + 1);
+
+  while (*name)
+    {
+      if (*name == '_')
+        g_string_append (escaped_name, "__");
+      else
+        g_string_append_c (escaped_name, *name);
+      name++;
+    }
+
+  return g_string_free (escaped_name, FALSE);
+}
+
 static void
 fill_in_new_term_submenus (TerminalWindow *window)
 {
@@ -454,6 +530,7 @@ fill_in_new_term_submenus (TerminalWindow *window)
   while (tmp != NULL)
     {
       TerminalProfile *profile;
+      char *escaped_name;
       
       profile = tmp->data;
       
@@ -461,16 +538,16 @@ fill_in_new_term_submenus (TerminalWindow *window)
       g_object_ref (G_OBJECT (profile));
       g_object_ref (G_OBJECT (profile));
 
-      /* FIXME underscores in profile name result in badness */
+      escaped_name = escape_underscores (terminal_profile_get_visible_name (profile));
+
       if (i < 10)
-        str = g_strdup_printf (_("_%d. %s"),
-                               i, terminal_profile_get_visible_name (profile));
+        str = g_strdup_printf (_("_%d. %s"), i, escaped_name);
       else if (i < 36)
-        str = g_strdup_printf (_("_%c. %s"),
-                               ('A' + i - 10),
-                               terminal_profile_get_visible_name (profile));
+        str = g_strdup_printf (_("_%c. %s"), ('A' + i - 10), escaped_name);
       else
-        str = g_strdup (terminal_profile_get_visible_name (profile));
+        str = g_strdup (escaped_name);
+
+      g_free (escaped_name);
       
       /* item for new window */
       menu_item = gtk_menu_item_new_with_mnemonic (str);
@@ -509,27 +586,6 @@ fill_in_new_term_submenus (TerminalWindow *window)
   g_list_free (profiles);
 }
 
-static void
-update_active_encoding_name (TerminalWindow *window)
-{
-  GtkWidget *w;
-  const char *charset;
-  char *name;
-  
-  w = terminal_screen_get_widget (window->priv->active_term);
-  charset = terminal_widget_get_encoding (w);
-
-  name = terminal_encoding_get_name (charset);
-
-#if 0
-  /* doesn't make sense anymore */
-  gtk_label_set_text (GTK_LABEL (gtk_bin_get_child (GTK_BIN (window->priv->encoding_menuitem))),
-                      name);
-#endif
-  
-  g_free (name);
-}
-
 static void
 fill_in_encoding_menu (TerminalWindow *window)
 {
@@ -560,8 +616,6 @@ fill_in_encoding_menu (TerminalWindow *window)
 
   w = terminal_screen_get_widget (window->priv->active_term);
   charset = terminal_widget_get_encoding (w);
-
-  update_active_encoding_name (window);
   
   group = NULL;
   encodings = terminal_get_active_encodings ();
@@ -647,8 +701,13 @@ terminal_window_init (TerminalWindow *window)
   GtkWidget *mi;
   GtkWidget *menu;
   GtkAccelGroup *accel_group;
+  GtkWidget *copy_image, *paste_image, *contents_image, *about_image;
+  GError *error;
+  gboolean use_image;
+
   
   gtk_window_set_title (GTK_WINDOW (window), _("Terminal"));
+  client = gconf_client_get_default();
   
   window->priv = g_new0 (TerminalWindowPrivate, 1);
   window->priv->terms = NULL;
@@ -734,11 +793,16 @@ terminal_window_init (TerminalWindow *window)
                            GTK_STOCK_COPY, ACCEL_PATH_COPY,
                            G_CALLBACK (copy_callback),
                            window);
+  window->priv->copy_notify_id = notify;
+  copy_image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (window->priv->copy_menuitem));
+
   window->priv->paste_menuitem =
     append_stock_menuitem (menu,
                            GTK_STOCK_PASTE, ACCEL_PATH_PASTE,
                            G_CALLBACK (paste_callback),
                            window);
+  window->priv->paste_notify_id = notify;
+  paste_image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (window->priv->paste_menuitem));
 
   mi = gtk_separator_menu_item_new ();
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
@@ -747,7 +811,7 @@ terminal_window_init (TerminalWindow *window)
     append_menuitem (menu, _("C_urrent Profile..."), NULL,
                      G_CALLBACK (edit_configuration_callback), window);
 
-  append_menuitem (menu, _("_Keybindings..."), NULL,
+  append_menuitem (menu, _("_Keyboard Shortcuts..."), NULL,
                    G_CALLBACK (edit_keybindings_callback), window);
 
   append_menuitem (menu, _("P_rofiles..."), NULL,
@@ -857,6 +921,8 @@ terminal_window_init (TerminalWindow *window)
 
   mi = append_stock_menuitem (menu, GTK_STOCK_HELP, NULL,
 			      G_CALLBACK (help_callback), window);
+  window->priv->help_notify_id = notify;
+  contents_image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (mi));
   set_menuitem_text (mi, _("_Contents"), FALSE);
 
   gtk_accel_map_add_entry (ACCEL_PATH_HELP, GDK_F1, 0);
@@ -866,6 +932,8 @@ terminal_window_init (TerminalWindow *window)
   
   mi = append_stock_menuitem (menu, GNOME_STOCK_ABOUT, NULL,
                               G_CALLBACK (about_callback), window);
+  window->priv->about_notify_id = notify;
+  about_image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (mi));
   set_menuitem_text (mi, _("_About"),
                      FALSE);
   
@@ -877,6 +945,23 @@ terminal_window_init (TerminalWindow *window)
   reset_menubar_labels (window);
   
   gtk_widget_show_all (window->priv->main_vbox);
+  error = NULL;
+  use_image = gconf_client_get_bool (client,
+                                     "/desktop/gnome/interface/menus_have_icons",
+                                     &error);
+  if (error)
+    {
+      g_printerr (_("There was an error loading config value for whether to use image in menus. (%s)\n"),error->message);
+      g_error_free (error);
+    }
+  else
+    {
+      menuitem_icon_visibility (copy_image, use_image);
+      menuitem_icon_visibility (paste_image, use_image);
+      menuitem_icon_visibility (contents_image, use_image);
+      menuitem_icon_visibility (about_image, use_image);
+    }
+
 }
 
 static void
@@ -904,6 +989,16 @@ terminal_window_finalize (GObject *object)
                               window->priv->notify_id);
   window->priv->notify_id = 0;
 
+  gconf_client_notify_remove (client, window->priv->copy_notify_id);
+  window->priv->copy_notify_id = 0;
+  gconf_client_notify_remove (client, window->priv->paste_notify_id);
+  window->priv->paste_notify_id = 0;
+  gconf_client_notify_remove (client, window->priv->help_notify_id);
+  window->priv->help_notify_id = 0;
+  gconf_client_notify_remove (client, window->priv->about_notify_id);
+  window->priv->about_notify_id = 0;
+
+
   if (window->priv->conf)
     g_object_unref (G_OBJECT (window->priv->conf));
   
@@ -1155,13 +1250,6 @@ selection_changed_callback (TerminalScreen *screen,
   update_copy_sensitivity (window);
 }
 
-static void
-encoding_changed_callback (TerminalScreen *screen,
-			  TerminalWindow *window)
-{
-  update_active_encoding_name (window);
-}
-
 void
 terminal_window_add_screen (TerminalWindow *window,
                             TerminalScreen *screen)
@@ -1215,11 +1303,6 @@ terminal_window_add_screen (TerminalWindow *window,
                     G_CALLBACK (selection_changed_callback),
                     window);
   
-  g_signal_connect (G_OBJECT (screen),
-                    "encoding_changed",
-                    G_CALLBACK (encoding_changed_callback),
-                    window);
-  
   term = terminal_screen_get_widget (screen);
 
   gtk_box_pack_start (GTK_BOX (hbox),
@@ -1940,6 +2023,7 @@ reset_tab_menuitems (TerminalWindow *window)
   int i;
   TerminalScreen *screen;
   GSList *group;
+  gboolean single_page;
   
   tmp = window->priv->tab_menuitems;
   while (tmp != NULL)
@@ -1951,6 +2035,8 @@ reset_tab_menuitems (TerminalWindow *window)
   g_list_free (window->priv->tab_menuitems);
   window->priv->tab_menuitems = NULL;
 
+  single_page = gtk_notebook_get_n_pages (GTK_NOTEBOOK (window->priv->notebook)) == 1;
+  
   group = NULL;
   i = 0;
   while (TRUE) /* should probably make us somewhat nervous */
@@ -1970,7 +2056,7 @@ reset_tab_menuitems (TerminalWindow *window)
                                                       terminal_screen_get_title (screen));
       group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (menu_item));
       
-      if (i < N_TABS_WITH_ACCEL)
+      if (i < N_TABS_WITH_ACCEL && !single_page)
         accel_path = g_strdup_printf (FORMAT_ACCEL_PATH_SWITCH_TO_TAB,
                                       i + 1);
       else
@@ -2091,13 +2177,15 @@ new_window_callback (GtkWidget      *menuitem,
   if (!terminal_profile_get_forgotten (profile))
     {
       char *name;
+      const char *dir;
       
       name = gdk_screen_make_display_name (gtk_widget_get_screen (menuitem));
+      dir = terminal_screen_get_working_dir (window->priv->active_term);
 
       terminal_app_new_terminal (terminal_app_get (),
                                  profile,
                                  NULL,
-                                 FALSE, FALSE, NULL, NULL, NULL, NULL, NULL, 1.0,
+                                 FALSE, FALSE, NULL, NULL, NULL, dir, NULL, 1.0,
                                  NULL, name, -1);
 
       g_free (name);
@@ -2117,10 +2205,14 @@ new_tab_callback (GtkWidget      *menuitem,
 
   if (!terminal_profile_get_forgotten (profile))
     {
+      const char *dir;
+
+      dir = terminal_screen_get_working_dir (window->priv->active_term);
+
       terminal_app_new_terminal (terminal_app_get (),
                                  profile,
                                  window,
-                                 FALSE, FALSE, NULL, NULL, NULL, NULL, NULL, 1.0,
+                                 FALSE, FALSE, NULL, NULL, NULL, dir, NULL, 1.0,
                                  NULL, NULL, -1);
     }
 }
@@ -2244,30 +2336,11 @@ fullscreen_callback (GtkWidget      *menuitem,
     {
       static GtkWidget *no_fullscreen_dialog = NULL;
 
-      if (no_fullscreen_dialog == NULL)
-        {
-          no_fullscreen_dialog =
-            gtk_message_dialog_new (GTK_WINDOW (window),
-                                    GTK_DIALOG_DESTROY_WITH_PARENT,
-                                    GTK_MESSAGE_ERROR,
-                                    GTK_BUTTONS_CLOSE,
+      terminal_util_show_error_dialog (GTK_WINDOW (window), &no_fullscreen_dialog,
                                     _("Your current desktop environment does not support the full screen feature.\n (In technical terms, you need a window manager with support for the _NET_WM_STATE_FULLSCREEN property.)"));
-          
-          g_object_add_weak_pointer (G_OBJECT (no_fullscreen_dialog),
-                                     (void**) &no_fullscreen_dialog);
-
-          g_signal_connect (G_OBJECT (no_fullscreen_dialog), "response",
-                            G_CALLBACK (gtk_widget_destroy),
-                            NULL);
-        }
-      
-      gtk_window_present (GTK_WINDOW (no_fullscreen_dialog));
-
-      return;
     }
-  
-  terminal_window_set_fullscreen (window,
-                                  !terminal_window_get_fullscreen (window));
+  else 
+    terminal_window_set_fullscreen (window, !terminal_window_get_fullscreen (window));
 }
 
 static double zoom_factors[] = {
@@ -2419,8 +2492,6 @@ change_encoding_callback (GtkWidget      *menu_item,
 
   widget = terminal_screen_get_widget (window->priv->active_term);
   terminal_widget_set_encoding (widget, charset);
-
-  update_active_encoding_name (window);
 }
 
 static void
@@ -2501,23 +2572,8 @@ help_callback (GtkWidget      *menuitem,
 
   if (err != NULL)
     {
-      GtkWidget *dialog;
-      
-      dialog = gtk_message_dialog_new (GTK_WINDOW (window),
-                                       GTK_DIALOG_DESTROY_WITH_PARENT,
-                                       GTK_MESSAGE_ERROR,
-                                       GTK_BUTTONS_CLOSE,
-                                       _("There was an error displaying help: %s"),
-                                       err->message);
-      
-      g_signal_connect (G_OBJECT (dialog), "response",
-                        G_CALLBACK (gtk_widget_destroy),
-                        NULL);
-      
-      gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-      
-      gtk_widget_show (dialog);
-
+      terminal_util_show_error_dialog (GTK_WINDOW (window), NULL,
+                                       _("There was an error displaying help: %s"), err->message);
       g_error_free (err);
     }
 }
@@ -2550,7 +2606,7 @@ about_callback (GtkWidget      *menuitem,
   g_free(file);
 
   about = gnome_about_new (PACKAGE, VERSION,
-                           _("Copyright 2002 Havoc Pennington"),
+                           "Copyright \xc2\xa9 2002 Havoc Pennington",
                            _("GNOME Terminal"),
                            (const char **)authors,
                            (const char **)documenters,
